{% include "html/utils/start.html"%}

    {% if consensus_summary is defined %}
    {% set step_summary = consensus_summary %}
    {% include "html/utils/step.html"%}
    {% endif %}

    {% if star_fusion_summary is defined %}
    {% set step_summary = star_fusion_summary %}
    {% include "html/utils/step.html"%}
    {% endif %}

    {% if count_fusion_summary is defined %}
    {% set step_summary = count_fusion_summary %}
    {% include "html/utils/step.html"%}
    {% endif %}

    {% if filter_fusion_summary is defined %}
    {% set step_summary = filter_fusion_summary %}
    {% include "html/utils/step.html"%}
    {% endif %}

    {% if analysis_fusion_summary is defined %}
    {% set step_summary = analysis_fusion_summary %}
    {% include "html/fusion/analysis_fusion.html"%}
    {% endif %}


{% include "html/utils/end.html"%}