
<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
    <h2>{{ step_summary.display_title}}   <i class="icon-help-circled" onClick="toggle1(this.parentNode.parentNode)" style="cursor:pointer;"></i></h2>
    <div class="box">

        {% include "html/utils/help_info.html" %}

        <div style="float: left; margin-left: 5%; margin-top: 1%;margin-right:5%; width: 90%">
          {{step_summary.substitution_box|safe}}
        </div>



    <div class="clear" ></div>
    </div>
</div>