<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
    <h2>{{step_summary.display_title}}    <i class="icon-help-circled" onClick="toggle1(this.parentNode.parentNode)" style="cursor:pointer;"></i></h2>
    <div class="box">
        {% include "html/utils/help_info.html" %}
        {% include "html/utils/col1_metrics.html" %}

        <div id="myDivUMI" style="float: left; margin-left: 3%; margin-top: 1%;margin-right:0%; width: 47%">
            {{ step_summary.chart|safe }}
        </div>

        {% include "html/utils/table_dict.html" %}

        <div class="clear" ></div>
    </div>
  </div>