{% include "html/utils/start.html"%}

    {% if mapping_tag_summary is defined %}
    {% set step_summary = mapping_tag_summary %}
    {% include "html/utils/step.html" %}
    {% endif %}

    {% if count_tag_summary is defined %}
    {% set step_summary = count_tag_summary %}
    {% include "html/utils/step.html"%}
    {% endif %}

    {% if analysis_tag_summary is defined %}
    {% set step_summary = analysis_tag_summary %}
    {% include "html/sweetseq/analysis_tag.html"%}
    {% endif %}

{% include "html/utils/end.html"%}