    <div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
      <h2>Mapping    <i class="icon-help-circled" onClick="toggle1(this.parentNode.parentNode)" style="cursor:pointer;"></i></h2>

      <div class="box">
        <div class="description" style="display: none;">
        </div>
        <table style="float: left; margin-left: 0%; margin-right:3%; width: 47%">
          {% for item in mapping_tag_summary %}
            {% if loop.index <= (loop.length+1)/2 %}
            <tr>
              {% for i in item %} 
              <td>{{ i|e }}</td>
              {% endfor %}
            </tr>
            {% endif %}
          {% endfor %}
        </table>

        <table style="float: left; margin-left: 3%; margin-right:0%; width: 47%">
          {% for item in mapping_tag_summary %}
            {% if loop.index > (loop.length+1)/2  %}
            <tr>
              {% for i in item %} 
              <td>{{ i|e }}</td>
              {% endfor %}
            </tr>
            {% endif %}
          {% endfor %}
        </table>
        <div class="clear" ></div>
      </div>
    </div>

