{% include "html/utils/start.html"%}

    {% if star_summary is defined %}
    {% set step_summary = star_summary %}
    {% include "html/utils/step.html" %}
    {% endif %}

    {% if featureCounts_summary is defined%}
    {% set step_summary = featureCounts_summary %}
    {% include "html/utils/step.html" %}
    {% endif %}

    {% if count_summary is defined %}
    {% set step_summary = count_summary %}
    {% include "html/bulk_rna/count.html"%}
    {% endif %}



{% include "html/utils/end.html"%}

