<!DOCTYPE html>
<html lang="en">

    {% include "html/utils/head.html"%}
    <body>
        <header class="navbar navbar-fixed-top" style="background: #ffffff">
        <div class="navbar-brand">
            {% include "html/common/logo.html"%}
        </div>
        </header>

        <div class="container">
            <h1 align="center">CeleScope Report</h1>

            {% include "html/utils/preprocessing.html"%}