{% include "html/utils/start.html"%}
    {% if mapping_tag_summary is defined %}
    {% set step_summary = mapping_tag_summary %}
    {% include "html/utils/step.html" %}
    {% endif %}

    {% if count_cite_summary is defined %}
    {% set step_summary = count_cite_summary %}
    {% include "html/utils/step.html" %}
    {% endif %}
      
    {% if analysis_cite_summary is defined %} 
    {% set step_summary = analysis_cite_summary %}
    {% include "html/citeseq/analysis_cite.html" %}
    {% endif %}
    
{% include "html/utils/end.html"%}
